/**
 * In Java gibt es Datenstrukturen in denen jedes Element genau einen Vorgänger und einen Nachfolger besitzt. Dazu gehören eindimensionale Arrays, Queues, Deques
 * und Stacks. In der folgenden Abbildung ist eine Datenstruktur zu sehen, welche dieselbe Eigenschaft aufweist, jedoch weder einen Anfang noch ein Ende besitzt.
 * Es handelt sich um eine einfach verkettete, zyklische Liste.
 *
 * Startknoten: clist[0] --- 1-->2-->3-->4-->5-->6 -- (Aktueller Knoten clist[1]
 *                           ^                       |
 *                           |                       |
 *                            ------------------------
 *
 * Diese Liste wird im beigefügten Programm in der main-Methode generiert. Zur Verwaltung der Liste dient ein Array namens clist, dessen
 * erstes Element auf einen definierten Startknoten der Liste verweist, sowie eine Variable list_length in der die Anzahl der Knoten der Liste gespeichert ist.
 *
 * Zusätzlich zu diesen Daten wird für unsere heutige Anwendung stets genau einer der Listenknoten als "aktuelle Position" angesehen. Eine Referenz auf ihn ist im
 * zweiten Element des Arrays clist zu finden.
 *
 * Die Methode printList gestattet es, eine derartige zyklische Liste mit beliebiger Länge am Bildschirm darzustellen. Dabei erhalten wir für obige Liste die
 * Ausgabe: [1->2->3->4->5->*6*-|] (1)
 *
 * In jeder derartigen Ausgabe ist der aktuelle Knoten mit zwei Sternsymbolen markiert. Das Nachfolgeelement des "letzten" Listenknoten ist in runden Klammern
 * am Ende der Ausgabe zu sehen.
 *
 * In der main-Methode wird die Methode transformList 8Mal aufgerufen. Ziel dieser Aufrufe ist es, das folgende Verhalten zu realisieren (Jeder Methodenaufruf
 * entspricht genau einem Schritt dieses Verfahrens):
 * Das Verfahren realisiert einzelne Schritte, wobei jeder Schritt in die Teile 1 und 2 unterteilt ist. Für Teil 2 wird abwechselnd Teil 2a und Teil 2b
 * ausgefürht
 *
 * Teil1: Wandere vom akteuellenKnoten genau vier Knoten in der Liste weiter. Der so erreichte Knoten soll der neue aktuelle Knoten sein.
 * Teil2: Führe entweder Teil 2a oder Teil 2b aus (wechselweise):
 * Teil 2a(Dritter Parameter von transormList ist true): Dupliziere den aktuellen Knoten und füge den neuen direkt danch ein.
 * Teil 2b(Dritter Parameter von transformList ist false): Lösche den aktuellen Knoten und betrachte den Nachfolger vom gelöschten Knoten als neuen aktuellen Knoten.
 *
 * Vervollständigen Sie die Methode transformList im beigefügten Programmcode, sodass mit ihr genau dieses Verfahren realisiert werden kann. Die zu ergänzenden
 * Stücke werden im Code mit Teil A bis F bezeichnet.
 *
 * Bei richtiger Implementierung soll das gesamte Programm nach der Ausgabe vom Ende der vorigen Seite genau den Block
 *
 * [1->2->3->4->5->*6*-|] (1)
 *
 * [1->2->3->*4*->4->5->6-|] (1)
 * [*2*->3->4->4->5->6-|] (2)
 *
 * [2->3->4->4->*5*->5->6-|] (2)
 * [2->*4*->4->5->5->6-|] (2)
 *
 * [2->4->4->5->5->*6*->6-|] (2)
 * [2->4->*5*->5->6->6-|] (2)
 *
 * [*2*->2->4->5->5->6->6-|] (2)
 * [2->2->4->5->*6*->6-|] (2)
 *
 * anzeigen.
 *
 *
 */
public class AdHoc6 {

    public int elem;
    public AdHoc6 next;

    public AdHoc6(int elem_local) {
        this.elem = elem_local;
        this.next = null;
    }

    public static void printList(AdHoc6[] clist_local, int list_length_local) {
        if (list_length_local > 0) {
            AdHoc6 current_node = clist_local[0];

            if (current_node  == clist_local[1]) {
                System.out.print("[" + "*" + current_node.elem + "*");
            }
            else {
                System.out.print("[" + current_node.elem);
            }

            for (int i = 1; i <= list_length_local - 1; i++) {
                current_node = current_node.next;

                if (current_node  == clist_local[1]) {
                    System.out.print("->" + "*" + current_node.elem + "*");
                }
                else {
                    System.out.print("->" + current_node.elem);
                }
            }

            System.out.println("-|] (" + current_node.next.elem + ")");
        }
    }

    public static int transformList(AdHoc6[] clist_local, int list_length_local, boolean is_duplicate_node) {
        AdHoc6 start_node = clist_local[0];
        AdHoc6 current_node = clist_local[1];

        current_node = current_node.next;
        current_node = current_node.next;
        current_node = current_node.next;
        // Nur drei Schritte durchgefuehrt !

        if (is_duplicate_node) {  // Dupliziere den Knoten current_node.next
            // ******************************
            // Ihre Implementierung: Teil A *
            // ******************************
            list_length_local++;
            // clist_local[1] = // Ihre Implementierung: Teil B
        }
        else {  // Loesche den Knoten current_node.next
            // if (/* Ihre Implementierung: Teil C */) {  // Loesche den Startknoten
            // ******************************
            // Ihre Implementierung: Teil D *
            // ******************************
            // }

            // ******************************
            // Ihre Implementierung: Teil E *
            // ******************************

            list_length_local--;
            // clist_local[1] = // Ihre Implementierung: Teil F
        }

        return list_length_local;
    }

    public static void main(String[] args) {
        AdHoc6[] clist = new AdHoc6[2];  // Erster Knoten, aktueller Knoten
        int list_length = 6;

        AdHoc6 n1 = new AdHoc6(1);
        clist[0] = n1;

        AdHoc6 n2 = new AdHoc6(2);
        n1.next = n2;

        AdHoc6 n3 = new AdHoc6(3);
        n2.next = n3;

        AdHoc6 n4 = new AdHoc6(4);
        n3.next = n4;

        AdHoc6 n5 = new AdHoc6(5);
        n4.next = n5;

        AdHoc6 n6 = new AdHoc6(6);
        n5.next = n6;

        clist[1] = n6;

        n6.next = n1;  // Eine zyklische Liste

        printList(clist, list_length);
        System.out.println();

        for (int iter = 1; iter <= 4; iter++) {
            list_length = transformList(clist, list_length, true);
            printList(clist, list_length);
            list_length = transformList(clist, list_length, false);
            printList(clist, list_length);
            System.out.println();
        }
    }
}
